--- cupsys-1.1.14/systemv/cupsaddsmb.c	2007-12-30 14:53:07.000000000 +0100
+++ cups-1.1.x_r2218/systemv/cupsaddsmb.c	2007-12-30 15:10:21.000000000 +0100
@@ -1,5 +1,5 @@
 /*
- * "$Id: cupsaddsmb.c,v 1.4 2002/02/12 18:47:13 mike Exp $"
+ * "$Id: cupsaddsmb.c 2218 2002-03-14 15:09:08Z mike $"
  *
  *   "cupsaddsmb" command for the Common UNIX Printing System (CUPS).
  *
@@ -43,7 +43,9 @@
  * Local globals...
  */
 
-int	Verbosity = 0;
+int		Verbosity = 0;
+const char	*SAMBAUser,
+		*SAMBAServer;
 
 
 /*
@@ -52,6 +54,7 @@
 
 int	do_samba_command(const char *, const char *);
 int	export_dest(const char *);
+void	usage();
 
 
 /*
@@ -64,51 +67,88 @@
 {
   int	i, j;		/* Looping vars */
   int	status;		/* Status from export_dest() */
+  int	export_all;	/* Export all printers? */
   int	num_printers;	/* Number of printers */
   char	**printers;	/* Printers */
 
 
+ /*
+  * Parse command-line arguments...
+  */
+
+  export_all = 0;
+
+  SAMBAUser   = cupsUser();
+  SAMBAServer = NULL;
+
   for (i = 1; i < argc; i ++)
     if (strcmp(argv[i], "-a") == 0)
+      export_all = 1;
+    else if (strcmp(argv[i], "-U") == 0)
     {
-      num_printers = cupsGetPrinters(&printers);
-
-      for (j = 0, status = 0; j < num_printers; j ++)
-        if ((status = export_dest(printers[j])) != 0)
-	  break;
-
-      for (j = 0; j < num_printers; j ++)
-        free(printers[j]);
+      i ++;
+      if (i >= argc)
+        usage();
 
-      if (num_printers)
-        free(printers);
+      SAMBAUser = argv[i];
+    }
+    else if (strcmp(argv[i], "-H") == 0)
+    {
+      i ++;
+      if (i >= argc)
+        usage();
 
-      if (status)
-        return (status);
+      SAMBAServer = argv[i];
     }
-    else if (strcmp(argv[i], "-U") == 0)
+    else if (strcmp(argv[i], "-h") == 0)
     {
       i ++;
       if (i >= argc)
-      {
-	puts("Usage: cupsaddsmb [-a] [-U user] [-v] [printer1 ... printerN]");
-	return (1);
-      }
+        usage();
 
-      cupsSetUser(argv[i]);
+      cupsSetServer(argv[i]);
     }
     else if (strcmp(argv[i], "-v") == 0)
       Verbosity = 1;
     else if (argv[i][0] != '-')
     {
+      if (SAMBAServer == NULL)
+	SAMBAServer = cupsServer();
+
       if ((status = export_dest(argv[i])) != 0)
 	return (status);
     }
     else
-    {
-      puts("Usage: cupsaddsmb [-a] [-U user] [-v] [printer1 ... printerN]");
-      return (1);
-    }
+      usage();
+
+ /*
+  * See if the user specified "-a"...
+  */
+
+  if (export_all)
+  {
+   /*
+    * Export all printers...
+    */
+
+    if (SAMBAServer == NULL)
+      SAMBAServer = cupsServer();
+
+    num_printers = cupsGetPrinters(&printers);
+
+    for (j = 0, status = 0; j < num_printers; j ++)
+      if ((status = export_dest(printers[j])) != 0)
+	break;
+
+    for (j = 0; j < num_printers; j ++)
+      free(printers[j]);
+
+    if (num_printers)
+      free(printers);
+
+    if (status)
+      return (status);
+  }
 
   return (0);
 }
@@ -134,20 +174,20 @@
     {
       snprintf(temp, sizeof(temp),
                "Password for %s required to access %s via SAMBA: ",
-	       cupsUser(), cupsServer());
+	       SAMBAUser, SAMBAServer);
 
       if ((p = cupsGetPassword(temp)) == NULL)
 	break;
     }
 
     snprintf(temp, sizeof(temp), "%s -N -U\'%s%%%s\' -c \'%s\'",
-             command, cupsUser(), p, subcmd);
+             command, SAMBAUser, p, subcmd);
 
     if (Verbosity)
       printf("Running command: %s\n", temp);
     else
     {
-      strncat(temp, " </dev/null >/dev/null 2>/dev/null", sizeof(temp) - strlen(temp) - 1);
+      strncat(temp, " </dev/null >/dev/null 2>/dev/null", sizeof(temp) - 1);
       temp[sizeof(temp) - 1] = '\0';
     }
 
@@ -156,7 +196,10 @@
       if (Verbosity)
         puts("");
 
-      p = NULL;
+      if (p[0])
+        p = NULL;
+      else
+        break;
     }
     else
     {
@@ -196,7 +239,7 @@
   }
 
   /* Do the smbclient commands needed for the Windows drivers... */
-  snprintf(command, sizeof(command), "smbclient //%s/print\\$", cupsServer());
+  snprintf(command, sizeof(command), "smbclient //%s/print\\$", SAMBAServer);
 
   snprintf(subcmd, sizeof(subcmd),
            "mkdir W32X86;"
@@ -241,7 +284,7 @@
            "adddriver \"Windows NT x86\" \"%s:ADOBEPS5.DLL:%s.PPD:ADOBEPSU.DLL:ADOBEPSU.HLP:NULL:RAW:NULL\"",
 	   dest, dest);
 
-  snprintf(command, sizeof(command), "rpcclient %s", cupsServer());
+  snprintf(command, sizeof(command), "rpcclient %s", SAMBAServer);
 
   if ((status = do_samba_command(command, subcmd)) != 0)
   {
@@ -276,5 +319,25 @@
 
 
 /*
- * End of "$Id: cupsaddsmb.c,v 1.4 2002/02/12 18:47:13 mike Exp $".
+ * 'usage()' - Show program usage and exit...
+ */
+
+void
+usage()
+{
+  puts("Usage: cupsaddsmb [options] printer1 ... printerN");
+  puts("       cupsaddsmb [options] -a");
+  puts("");
+  puts("Options:");
+  puts("  -H samba-server  Use the named SAMBA server");
+  puts("  -U samba-user    Authenticate using the named SAMBA user");
+  puts("  -a               Export all printers");
+  puts("  -h cups-server   Use the named CUPS server");
+  puts("  -v               Be verbose (show commands)");
+  exit(1);
+}
+
+
+/*
+ * End of "$Id: cupsaddsmb.c 2218 2002-03-14 15:09:08Z mike $".
  */
